package FilterProxy::BlockBanner;

# Filtering module
# need vars/methods to check
# uri regexp (filter if uri matches this regexp)
# filter method (passed file as scalar)

use strict;
use vars qw($VERSION $CONFIG);
$VERSION = 0.04;
push @FilterProxy::MODULES, "BlockBanner";
$FilterProxy::BlockBanner::CONFIG = {};
$CONFIG->{mime_types} = ["text/html"];
$CONFIG->{order} = 4;
*debug = \&FilterProxy::debug;
my($whitespace) = " |\s|\015|\013|\r|\f|\t|\n";
my($dontcare) = "$whitespace|&nbsp;|<[bh]r[^>]*>|<p[^>]*>|<\!--(.*?)-->";
my($dontcarescript) = "<(?:$whitespace)*script[^>]*?>(.*?)<(?:$whitespace)*/script[^>]*?>";
my($dontcareilayer) = "<(?:$whitespace)*ilayer[^>]*?>(.*?)<(?:$whitespace)*/ilayer[^>]*?>";
my($dontcarelayer) = "<(?:$whitespace)*layer[^>]*?>(.*?)<(?:$whitespace)*/layer[^>]*?>";
my($basicad) = "(?:<(?:$whitespace)*a(?:$whitespace)[^>]*href[^>]*>.*?<(?:$whitespace)*\/a(?:$whitespace)*>|<(?:$whitespace)*layer(?:$whitespace)[^>]*>.*?<(?:$whitespace)*/layer[^>]*>|<(?:$whitespace)*ilayer(?:$whitespace)[^>]*>.*?<(?:$whitespace)*/ilayer[^>]*>|<(?:$whitespace)*frame(?:$whitespace)[^>]*>.*?<(?:$whitespace)*/frame[^>]*>|<(?:$whitespace)*iframe(?:$whitespace)[^>]*>.*?<(?:$whitespace)*/iframe[^>]*>)";

sub filter {
  my($req, $res, $siteconfig) = @_;
  my($numbasicads) = 0;
  my($numads) = 0;
  my($matchpos) = 0;
  my($endpos) = 0;
  pos(${$res->content_ref}) = 0;

  my($adregexp) = "";
  my($site);
  foreach $site (@$siteconfig) {
    $site->{'adregexp'} = [] unless defined $site->{'adregexp'};
    $adregexp .= join("|", @{$site->{'adregexp'}}) . "|";
  }
  $adregexp =~ s/^\|+//;
  $adregexp =~ s/\|+$//;
  $adregexp = "(" . $adregexp . ")";
  return if $adregexp eq "()";

BASICAD:
  while(${$res->content_ref} =~ /\G.*?($basicad)/sig) {
    my($ad) = $1;
    $numbasicads++;
    next BASICAD unless($ad =~ /(img|src)/si);
    next BASICAD unless($ad =~ /$adregexp/si);
    $endpos     = pos(${$res->content_ref});
    $matchpos   = pos(${$res->content_ref}) - length $ad;
    $numads++;
# suck in surrounding useless tags
    my($tag);
    my($tmpmatchpos) = $matchpos;
    my($tmpendpos) = $endpos;
ENCLOSINGTAG:
    while(1) {
      pos(${$res->content_ref}) = $endpos;
      last ENCLOSINGTAG unless ${$res->content_ref} =~ /\G(?:$dontcare)*<\s*\/([a-z]+)[^>]*>(?:$dontcare)*/sig;
      $tag = $2; # note 1 parenthesis in $dontcare
      my($leadcomment, $trailcomment) = ($1, $3);
      last ENCLOSINGTAG if(defined $leadcomment && $leadcomment =~ /<!--|-->/);
      last ENCLOSINGTAG if(defined $trailcomment && $trailcomment =~ /<!--|-->/);
      $tmpendpos = pos(${$res->content_ref});
      $tmpmatchpos = $matchpos;
      do {
        do {
          do {
            $tmpmatchpos--;
          } while((substr(${$res->content_ref}, $tmpmatchpos, 1) ne '<') && $tmpmatchpos>0);
        } while(substr(${$res->content_ref}, $tmpmatchpos, $matchpos-$tmpmatchpos) =~ /^(?:$dontcare)*$/si);
        pos(${$res->content_ref}) = $tmpmatchpos;
      } while(${$res->content_ref} !~ /\G(<\s*\/?[a-z]*[^>]*?>)/sig); 
# find previous tag
      pos(${$res->content_ref}) = $tmpmatchpos;
      last ENCLOSINGTAG unless ${$res->content_ref} =~ /\G<\s*$tag[^>]*>/sig;
      $matchpos = $tmpmatchpos;
      $endpos = $tmpendpos;
      pos(${$res->content_ref}) = $tmpendpos;
      if($tag =~ /noscript|nolayer/si) {  # Try to find corresponding <script>
        my($dontcareopp, $dontcaretag);
        if($tag =~ /noscript/si) {
          $dontcareopp = $dontcarescript;
          $dontcaretag = "script";
        }
        if($tag =~ /nolayer/si) {
          $dontcareopp = "(?:" . $dontcarelayer . "|" . $dontcareilayer . ")";
          $dontcaretag = "(?:ilayer|layer)";
        }
        # Corresponding tag trails
        if(${$res->content_ref} =~ /\G(?:$dontcare)*?$dontcareopp(?:$dontcare)*/sig) {
          $endpos = pos(${$res->content_ref});
        } else { # corresponding tag leads
          do {
            do {
              $tmpmatchpos--;
            } while((substr(${$res->content_ref}, $tmpmatchpos, 1) ne '<') && $tmpmatchpos>0);
          } while(substr(${$res->content_ref}, $tmpmatchpos, $matchpos-$tmpmatchpos) =~ /^(?:$dontcare)*$/si);
          pos(${$res->content_ref}) = $tmpmatchpos;
          while(${$res->content_ref} =~ /\G<\s*\/$dontcaretag[^>]*?>/sig) { # preceeding tag is </script>
            do { # rewind to previous <script> tag
              do {
                $tmpmatchpos--;
              } while((substr(${$res->content_ref}, $tmpmatchpos, 1) ne '<') && $tmpmatchpos>0);
            } while(substr(${$res->content_ref}, $tmpmatchpos, $matchpos-$tmpmatchpos) !~ /^<\s*$dontcaretag[^>]*>/si 
                && $tmpmatchpos>0);
            if($tmpmatchpos > 0) {
              $matchpos = $tmpmatchpos;
            }
            do { # rewind to previous tag, check if it is </script>
              do {
                $tmpmatchpos--;
              } while((substr(${$res->content_ref}, $tmpmatchpos, 1) ne '<') && $tmpmatchpos>0);
            } while(substr(${$res->content_ref}, $tmpmatchpos, $matchpos-$tmpmatchpos) =~ /^(?:$dontcare)*$/si);
            pos(${$res->content_ref}) = $tmpmatchpos;
          }
        }
      }
    }
    debug("  Stripping ad:\n", substr(${$res->content_ref}, $matchpos, $endpos-$matchpos));
    substr(${$res->content_ref}, $matchpos, $endpos-$matchpos) = "";
    pos(${$res->content_ref}) = $matchpos;
  }
  debug("  BlockBanner found $numbasicads basic ads, $numads real ads.\n");
}

sub Config {
  my($req, $cgi, $siteconfig) = @_;
  my($message) = "";

  if($cgi->param('adregexp')) {
    $siteconfig->{'adregexp'} = [split(/\s*[\r\n]+/, $cgi->param('adregexp'))];
    $message = "adregexp successfully updated";
  }
  return $message; # message printed to user
}

1;
