# Compress FilterProxy Module

package FilterProxy::Compress;

use strict;
use vars qw($VERSION $CONFIG);
use Compress::Zlib;

push @FilterProxy::MODULES, "Compress"; # add myself to FilterProxy's list of known modules
$VERSION = 0.01;
$CONFIG = {};
$CONFIG->{order} = 9;
$CONFIG->{mime_types} = [ # types which compress well.
    "text/html", 
    "text/plain",
    "application/postscript",
    "application/pdf", 
    "application/x-javascript"
];
$CONFIG->{min_size} = 1024;     # Don't compress anything smaller than 1k.

*debug = \&FilterProxy::debug;

sub filter {
  my($req, $res, $siteconfig) = @_;
  return unless (defined $req) && (ref $req) && (defined $res) && (ref $res);
  return unless (length ${$res->content_ref} > $CONFIG->{min_size});
  return unless grep($res->content_type, $CONFIG->{mime_types});
  return unless (($res->content_encoding || "") !~ /\S/ 
             and ($req->header("accept-encoding") || "") =~ /gzip/);
  my($oldlength) = length ${$res->content_ref};

  $res->content(Compress::Zlib::memGzip($res->content_ref)); #will clobber file if fail
  if (defined ${$res->content_ref}) {
    $res->content_encoding("gzip");
    debug("  gzipping ", $req->uri, " from ".  $oldlength . " to " .
         (length ${$res->content_ref}) . " bytes\n");
  } else {
    debug("  gzip failed!  Lost data!\n");
  }
  return;
}

sub Config {
  my($req, $cgi, $siteconfig) = @_;

  if($cgi->param()) {
    if(defined $cgi->param('min_size')) {
      $CONFIG->{min_size} = $cgi->param('min_size');
    }
    if(defined $cgi->param('mime_types')) {
      $CONFIG->{mime_types} = [split(/\s*[\r\n]+/, $cgi->param('mime_types'))];
    }
  }
  return "It worked.\n";
}
